@@ -42,7 +42,7 @@
  */
 
 typedef unsigned renderer_t;
-enum renderer_e {GS = 0, PDFTOPS = 1, ACROREAD = 2};
+enum renderer_e {GS = 0, PDFTOPS = 1, ACROREAD = 2, PDFTOCAIRO = 3};
 
 /*
  * Local functions...
@@ -381,6 +381,8 @@ main(int  argc,				/* I - Number of command-line args */
       renderer = PDFTOPS;
     else if (strcasecmp(val, "acroread") == 0)
       renderer = ACROREAD;
+    else if (strcasecmp(val, "pdftocairo") == 0)
+      renderer = PDFTOCAIRO;
     else
       fprintf(stderr,
 	      "WARNING: Invalid value for \"pdftops-renderer\": \"%s\"\n", val);
@@ -456,6 +458,12 @@ main(int  argc,				/* I - Number of command-line args */
     pdf_argv[6] = (char *)"-sOUTPUTFILE=%stdout";
     pdf_argc    = 7;
   }
+  else if (renderer == PDFTOCAIRO)
+  {
+    pdf_argv[0] = (char *)"pdftocairo";
+    pdf_argv[1] = (char *)"-ps";
+    pdf_argc    = 2;
+  }
   else
   {
     pdf_argv[0] = (char *)"acroread";
@@ -478,6 +486,8 @@ main(int  argc,				/* I - Number of command-line args */
       }
       else if (renderer == GS)
 	pdf_argv[pdf_argc++] = (char *)"-dLanguageLevel=1";
+      else if (renderer == PDFTOCAIRO)
+        fprintf(stderr, "WARNING: Level 1 PostScript not supported by pdftocairo.");
       else
         fprintf(stderr, "WARNING: Level 1 PostScript not supported by acroread.");
     }
@@ -491,18 +501,26 @@ main(int  argc,				/* I - Number of command-line args */
       }
       else if (renderer == GS)
 	pdf_argv[pdf_argc++] = (char *)"-dLanguageLevel=2";
-      else
+      else /* PDFTOCAIRO || ACROREAD */
         pdf_argv[pdf_argc++] = (char *)"-level2";
     }
     else
     {
       if (renderer == PDFTOPS)
-        /* Do not emit PS Level 3 with Poppler, some HP PostScript printers
-	   do not like it. See https://bugs.launchpad.net/bugs/277404. */
-	pdf_argv[pdf_argc++] = (char *)"-level2";
+      {
+        /* Do not emit PS Level 3 with Poppler on HP PostScript laser printers
+	   as some do not like it. See https://bugs.launchpad.net/bugs/277404.*/
+	if (ppd->manufacturer &&
+	    (!strncasecmp(ppd->manufacturer, "HP", 2) ||
+	     !strncasecmp(ppd->manufacturer, "Hewlett-Packard", 15)) &&
+	    (strcasestr(ppd->nickname, "laserjet")))
+	  pdf_argv[pdf_argc++] = (char *)"-level2";
+	else
+	  pdf_argv[pdf_argc++] = (char *)"-level3";
+      }
       else if (renderer == GS)
 	pdf_argv[pdf_argc++] = (char *)"-dLanguageLevel=3";
-      else
+      else /* PDFTOCAIRO || ACROREAD */
         pdf_argv[pdf_argc++] = (char *)"-level3";
     }
 
@@ -551,7 +569,7 @@ main(int  argc,				/* I - Number of command-line args */
 	  orientation ^= 1;
       }
 
-      if (renderer == PDFTOPS)
+      if ((renderer == PDFTOPS) || (renderer == PDFTOCAIRO))
       {
 	if (orientation & 1)
 	{
@@ -605,7 +623,7 @@ main(int  argc,				/* I - Number of command-line args */
       }
     }
 #ifdef HAVE_POPPLER_PDFTOPS_WITH_ORIGPAGESIZES
-    else if (renderer == PDFTOPS)
+    else if ((renderer == PDFTOPS) || (renderer == PDFTOCAIRO))
     {
      /*
       *  Use the page sizes of the original PDF document, this way documents
@@ -678,7 +696,7 @@ main(int  argc,				/* I - Number of command-line args */
     while (xres > maxres)
       xres = xres / 2;
 
-  if (renderer == PDFTOPS)
+  if ((renderer == PDFTOPS) || (renderer == PDFTOCAIRO))
   {
 #ifdef HAVE_POPPLER_PDFTOPS_WITH_RESOLUTION
    /*
@@ -748,7 +766,7 @@ main(int  argc,				/* I - Number of command-line args */
   * of the printer's PostScript interpreter?
   */
 
-  if (renderer == PDFTOPS)
+  if ((renderer == PDFTOPS) || (renderer == PDFTOPS))
     need_post_proc = 0;
   else if (renderer == GS)
     need_post_proc =
@@ -800,14 +818,19 @@ main(int  argc,				/* I - Number of command-line args */
 
     if (renderer == PDFTOPS)
     {
-      execv(CUPS_POPPLER_PDFTOPS, pdf_argv);
+      execvp(CUPS_POPPLER_PDFTOPS, pdf_argv);
       perror("DEBUG: Unable to execute pdftops program");
     }
     else if (renderer == GS)
     {
-      execv(CUPS_GHOSTSCRIPT, pdf_argv);
+      execvp(CUPS_GHOSTSCRIPT, pdf_argv);
       perror("DEBUG: Unable to execute gs program");
     }
+    else if (renderer == PDFTOCAIRO)
+    {
+      execvp(CUPS_POPPLER_PDFTOCAIRO, pdf_argv);
+      perror("DEBUG: Unable to execute pdftocairo program");
+    }
     else
     {
       /*
@@ -820,7 +843,7 @@ main(int  argc,				/* I - Number of command-line args */
         close(fd);
       }
      
-      execv(CUPS_ACROREAD, pdf_argv);
+      execvp(CUPS_ACROREAD, pdf_argv);
       perror("DEBUG: Unable to execute acroread program");
     }
 
@@ -836,6 +859,8 @@ main(int  argc,				/* I - Number of command-line args */
       perror("DEBUG: Unable to execute pdftops program");
     else if (renderer == GS)
       perror("DEBUG: Unable to execute gs program");
+    else if (renderer == PDFTOCAIRO)
+      perror("DEBUG: Unable to execute pdftocairo program");
     else
       perror("DEBUG: Unable to execute acroread program");
 
@@ -1097,7 +1122,8 @@ main(int  argc,				/* I - Number of command-line args */
 	fprintf(stderr, "DEBUG: PID %d (%s) stopped with status %d!\n",
 		wait_pid,
 		wait_pid == pdf_pid ? (renderer == PDFTOPS ? "pdftops" :
-                (renderer == GS ? "gs" : "acroread")) :
+                (renderer == PDFTOCAIRO ? "pdftocairo" :
+		(renderer == GS ? "gs" : "acroread"))) :
 		(wait_pid == pstops_pid ? "pstops" : "Post-processing"),
 		exit_status);
       }
@@ -1107,7 +1133,8 @@ main(int  argc,				/* I - Number of command-line args */
 		"DEBUG: PID %d (%s) was terminated normally with signal %d!\n",
 		wait_pid,
 		wait_pid == pdf_pid ? (renderer == PDFTOPS ? "pdftops" :
-                (renderer == GS ? "gs" : "acroread")) :
+                (renderer == PDFTOCAIRO ? "pdftocairo" :
+                (renderer == GS ? "gs" : "acroread"))) :
 		(wait_pid == pstops_pid ? "pstops" : "Post-processing"),
 		exit_status);
       }
@@ -1117,7 +1144,8 @@ main(int  argc,				/* I - Number of command-line args */
 
 	fprintf(stderr, "DEBUG: PID %d (%s) crashed on signal %d!\n", wait_pid,
 		wait_pid == pdf_pid ? (renderer == PDFTOPS ? "pdftops" :
-                (renderer == GS ? "gs" : "acroread")) :
+                (renderer == PDFTOCAIRO ? "pdftocairo" :
+                (renderer == GS ? "gs" : "acroread"))) :
 		(wait_pid == pstops_pid ? "pstops" : "Post-processing"),
 		exit_status);
       }
@@ -1126,7 +1154,8 @@ main(int  argc,				/* I - Number of command-line args */
     {
       fprintf(stderr, "DEBUG: PID %d (%s) exited with no errors.\n", wait_pid,
 	      wait_pid == pdf_pid ? (renderer == PDFTOPS ? "pdftops" :
-              (renderer == GS ? "gs" : "acroread")) :
+              (renderer == PDFTOCAIRO ? "pdftocairo" :
+              (renderer == GS ? "gs" : "acroread"))) :
 	      (wait_pid == pstops_pid ? "pstops" : "Post-processing"));
     }
   }
